<?php
/*
  Plugin Name: debug-printr
  Plugin URI: http://www.iaps.ca/wordpress-plugins/debug-printr/
  Description: Useage: debug_printr($obj) in template.
  Author: Brett Farrell
  Version: 1.4.2
  Author URI: http://www.iaps.ca/wordpress-plugins/
 */

if (!function_exists('debug_printr')) {

    /**
     * Prints $obj data to the screen.
     * @param mixed $obj The variable you want to print_r.
     * @param bool $exit If you want to call exit() right after printing $obj.  Default = false.
     * @param string $file Fallback if debug_backtrace() is not available.  $obj filename, default = null.
     * @param string $line Fallback if debug_backtrace() is not available.  $obj line number, default = null.
     * @param bool $print_pre If you want to enclose $obj in <pre></pre>.  Default = true.
     */
    function debug_printr($obj, $exit = false, $file = null, $line = null, $print_pre = true) {
        debug_printr::print_r($obj, $exit, $file, $line, $print_pre);
    }

}

class debug_printr {

    const OPTION = 'debug_ips';
    const OPTION_USERS = 'debug_users';

    function __construct() {
        if (!is_admin())
            return;
        // set defaults
        register_activation_hook(__FILE__, [$this, 'activate']);
        // Add our menu item
        add_action('admin_menu', [$this, 'plugin_menu']);
        // Add Plugin-Page Links: Settings
        add_filter('plugin_action_links_' . plugin_basename(__FILE__), [$this, 'add_action_links']);
        // check post
        add_action('admin_init', [$this, 'check_post'], 10, 0);
    }

    function activate() {
        if (!get_option(self::OPTION)) {
            update_option(self::OPTION, json_encode(array("127.0.0.1", $_SERVER['REMOTE_ADDR'])));
            update_option(self::OPTION_USERS, json_encode(array(wp_get_current_user()->ID)));
        }
    }

    function plugin_menu() {
        add_options_page('Debug PrintR', 'Debug PrintR', 'manage_options', 'debug-printr', [$this, 'admin_settings']);
    }

    function admin_settings() {
        if (!current_user_can('manage_options')) {
            wp_die(__('You do not have sufficient permissions to access this page.'));
        }
        $editor_url = home_url() . '/wp-admin/plugin-editor.php?plugin=debug-printr%2Fplugin.php';
        $users = new WP_User_Query(['role' => 'Administrator']);
        ?>
        <style>#debug-ips textarea {min-height: 200px;}</style>
        <form action="" method="post" id="debug-ips">
            <?php wp_nonce_field('action_debug_printr_save_ips', 'debug_printr_save_ips') ?>
            <p><?= __('Append any debug-enabled IP addresses or hostnames to the list, one per line.') ?></p>
            <textarea name="ips"><?= implode("\r\n", json_decode(get_option(self::OPTION))) ?></textarea>
            <p><?= __('Select any debug-enabled WP Users - Administrators (CTRL + Click to select multiple):') ?></p>
            <select size="5" name="users[]">
                <?php foreach ($users->get_results() as $user): ?>
                    <option value="<?= $user->ID ?>" <?= in_array($user->ID, (array)json_decode(get_option(self::OPTION_USERS))) ? 'selected' : null ?>><?= $user->user_email ?></option>
                <?php endforeach; ?>
            </select>
            <input type="submit" class="button button-primary button-large" name="debug-printr" value="<?= __('Save') ?>" />
        </form>
        <p>Your IP: <strong><?= $_SERVER['REMOTE_ADDR'] ?></strong></p>
        <p>Debugging Enabled? <strong><?= var_export(self::enabled(), true) ?></strong></p>
        <p>Usage: <pre>&lt;?php debug_printr($var, $exit = false, $file = null, $line = null, $print_pre = true); ?&gt;</pre> or <pre>&lt;?php debug_printr::print_r($var, $exit = false, $file = null, $line = null, $print_pre = true); ?&gt;</pre></p>
        <p>For more information on function parameters, view code source: <a href="<?= $editor_url ?>"><?= $editor_url ?></a></p>
        <?php
    }

    function check_post() {
        if (!array_key_exists('debug-printr', $_POST) ||
                !strstr($_SERVER['REQUEST_URI'], '?page=debug-printr') ||
                !array_key_exists('debug_printr_save_ips', $_POST) ||
                !wp_verify_nonce($_POST['debug_printr_save_ips'], 'action_debug_printr_save_ips'))
            return;
        $ips = explode("\r\n", trim($_POST['ips']));
        for ($i = 0; $i < count($ips); $i++):
            $ip = $ips[$i];
            if (!filter_var($ip, FILTER_VALIDATE_IP) && !filter_var(gethostbyname($ip), FILTER_VALIDATE_IP))
                unset($ips[$i]);
        endfor;
        $ips = array_values($ips);
        if (json_decode(get_option(self::OPTION)) !== $ips)
            update_option(self::OPTION, json_encode($ips));
        update_option(self::OPTION_USERS, json_encode($_POST['users']));
    }

    function add_action_links($links) {
        // merge in our links first so they show first
        return array_merge(array('<a href="' . admin_url('admin.php?page=debug-printr') . '">Settings</a>'), $links);
    }

    /**
     * Prints $obj data to the screen.
     * @param mixed $obj The variable you want to print_r.
     * @param bool $exit If you want to call exit() right after printing $obj.  Default = false.
     * @param string $file Fallback if debug_backtrace() is not available.  $obj filename, default = null.
     * @param string $line Fallback if debug_backtrace() is not available.  $obj line number, default = null.
     * @param bool $print_pre If you want to enclose $obj in <pre></pre>.  Default = true.
     */
    public static function print_r($obj, $exit = false, $file = null, $line = null, $print_pre = true) {

        if (self::enabled()) {
            if ($print_pre)
                echo "<pre>\n";
            echo "-------------------------------------------------------------------------------\n";
            //print_r(debug_backtrace());
            //exit;
            if ($debug_backtrace = debug_backtrace()) {
                $backtrace_index = 0;
                if (@$debug_backtrace[$backtrace_index]['class'] == __CLASS__)
                    $backtrace_index = 1;
                $file = isset($debug_backtrace[$backtrace_index]['file']) ? $debug_backtrace[$backtrace_index]['file'] : $file;
                $line = isset($debug_backtrace[$backtrace_index]['line']) ? $debug_backtrace[$backtrace_index]['line'] : $line;
            }
            echo $file . ":[$line]\n\n";
            print_r($obj);
            echo "\n-------------------------------------------------------------------------------\n";
            if ($print_pre)
                echo "</pre>\n";
            if ($exit)
                exit('exited...');
        }
    }

    /**
     * Determines whether debugging is enabled based on the IPs provided.
     * @return boolean
     */
    public static function enabled() {
        if (in_array(wp_get_current_user()->ID, (array) json_decode(get_option(self::OPTION_USERS))))
            return true;
        $ips = (array) json_decode(get_option(self::OPTION));
        foreach ($ips as &$ip):
            if (filter_var($ip, FILTER_VALIDATE_IP))
                continue;
            if (filter_var(gethostbyname($ip), FILTER_VALIDATE_IP))
                $ip = gethostbyname($ip);
        endforeach;
        if (in_array($_SERVER["REMOTE_ADDR"], $ips))
            return true;
        return false;
    }

}

$debug_printr = new debug_printr();
